/*------------------------------------------------------------------------
--                                                                      --
--                     H E L I O S   N U C L E U S                      --
--                     ---------------------------                      --
--                                                                      --
--             Copyright (C) 1987, Perihelion Software Ltd.             --
--                        All Rights Reserved.                          --
--                                                                      --
-- memory.h								--
--                                                                      --
--	Kernel memory primitives					--
--                                                                      --
--	Author:  NHG 16/8/87						--
--                                                                      --
------------------------------------------------------------------------*/
/* SccsId: %W% %G% Copyright (C) 1987, Perihelion Software Ltd.		*/
/* RcsId: $Id: memory.h,v 1.3 90/11/06 16:30:02 paul Exp $ */

#ifndef __memory_h
#define __memory_h

#ifndef __helios_h
# include <helios.h>
#endif

#include <queue.h>

/* Memory block header */

typedef struct Memory {
	Node		Node;		/* node in current pool		*/
	WORD		Size;		/* block size + alloc bits	*/
	struct Pool	*Pool;		/* pointer to owning pool	*/
} Memory;

#define Memory_Size_BitMask	15	/* mask for alloc bits		*/
#define Memory_Size_BwdBit	2	/* alloc state of predecessor	*/
#define Memory_Size_FwdBit	1	/* alloc state of this block	*/
#define Memory_Size_Fast	4	/* marks fast ram carrier	*/

/* Memory pool structure */
typedef struct Pool {
	Node		Node;		/* queuing node			*/
	List		Memory;		/* list of blocks owned		*/
	WORD		Blocks;		/* number of blocks in pool	*/
	WORD		Size;		/* total size of pool		*/
	WORD		Max;		/* initial size of pool		*/
} Pool;

/* Memory allocation procedures */

PUBLIC void InitPool(Pool *);
PUBLIC void *AllocMem(WORD , Pool *);
PUBLIC word FreeMem(void *);
PUBLIC void FreeMemStop(void *);
PUBLIC void FreePool(Pool *);
PUBLIC bool InPool(void *addr, Pool *pool);
#ifdef __HELIOSARM
PUBLIC void *LowAllocMem(WORD size, Pool *);
PUBLIC bool ReAllocMem(WORD newsize, void *addr);
#endif

#ifndef __HELIOSARM
/* Fast Ram carrier */

typedef struct Carrier {
	byte		*Addr;		/* address of fast ram block	*/
	word		Size;		/* size of block		*/
} Carrier;

PUBLIC Carrier *AllocFast(word, Pool *);
#endif
	
#endif

/* -- End of memory.h */
