/*------------------------------------------------------------------------
--                                                                      --
--           H E L I O S   N E T W O R K I N G   S O F T W A R E	--
--           ---------------------------------------------------	--
--                                                                      --
--             Copyright (C) 1990, Perihelion Software Ltd.             --
--                        All Rights Reserved.                          --
--                                                                      --
-- rmlib.h								--
--                                                                      --
--	Header file defining the Resource Management library		--
--                                                                      --
--	Author:  BLV 1/5/90						--
--                                                                      --
------------------------------------------------------------------------*/

/* $Header: /usr/perihelion/Helios/include/RCS/rmlib.h,v 1.1 90/09/14 14:34:28 bart Exp $ */

#ifndef __rmlib_h
#define __rmlib_h

/**
*** The include files that are required
**/
#ifndef __helios_h
#include <helios.h>
#endif

#ifndef __queue_h
#include <queue.h>
#endif

#ifndef __syslib_h
#include <syslib.h>
#endif

#ifndef __RmStructs
typedef void	*RmProcessor;
typedef void	*RmNetwork;
typedef void	*RmTask;
typedef void	*RmTaskforce;
#endif
typedef char	*RmID;
typedef struct RmHardwareFacility {
	Node		Node;
	int		Type;	/* Reset driver, reset command, ... */
	int		NumberProcessors;
	void		*Device; 	/* for internal use */
	RmProcessor	Essential;	/* ditto */
	int		Spare[2];
	char		Name[64];	/* pa_ra.d, or -e /helios/net/tr_reset*/
	char		Option[64];	/* Processor name or driver option */
	RmProcessor	*Processors;
} RmHardwareFacility;

extern RmProcessor	RmNewProcessor(void);
extern int		RmFreeProcessor(RmProcessor);
extern int		RmGetProcessorMemory(RmProcessor);
extern int		RmSetProcessorMemory(RmProcessor, int);
extern const RmID	RmGetProcessorID(RmProcessor);
extern int		RmSetProcessorID(RmProcessor, RmID);
extern int		RmGetProcessorPurpose(RmProcessor);
extern int		RmSetProcessorPurpose(RmProcessor, int);
extern int		RmGetProcessorState(RmProcessor);
extern int		RmSetProcessorState(RmProcessor, int);
extern int		RmGetProcessorType(RmProcessor);
extern int		RmSetProcessorType(RmProcessor, int);
extern int		RmGetProcessorOwner(RmProcessor);
extern int		RmWhoAmI(void);
extern const char	*RmWhoIs(int);
extern const char	*RmGetProcessorNucleus(RmProcessor);
extern int		RmSetProcessorNucleus(RmProcessor, char *);
extern int		RmAddProcessorAttribute(RmProcessor, char *);
extern int		RmRemoveProcessorAttribute(RmProcessor, char *);
extern int		RmIsAProcessorAttribute(RmProcessor, char *);
extern int		RmCountProcessorAttributes(RmProcessor);
extern int		RmListProcessorAttributes(RmProcessor, char **);

extern RmTask		RmNewTask(void);
extern int		RmFreeTask(RmTask);
extern int		RmGetTaskMemory(RmTask);
extern int		RmSetTaskMemory(RmTask, int);
extern const RmID	RmGetTaskID(RmTask);
extern int		RmSetTaskID(RmTask, RmID);
extern int		RmGetTaskType(RmTask);
extern int		RmSetTaskType(RmTask, int);
extern int		RmSetTaskNative(RmTask);
extern bool		RmIsTaskNative(RmTask);
extern int		RmSetTaskNormal(RmTask);
extern int		RmSetTaskforceNative(RmTaskforce);
extern int		RmSetTaskforceNormal(RmTaskforce);
extern int		RmGetProgramType(char *);
extern const char	*RmGetTaskCode(RmTask);
extern int		RmSetTaskCode(RmTask, char *);
extern int		RmAddTaskAttribute(RmTask, char *);
extern int		RmRemoveTaskAttribute(RmTask, char *);
extern int		RmIsATaskAttribute(RmTask, char *);
extern int		RmCountTaskAttributes(RmTask);
extern int		RmListTaskAttributes(RmTask, char **);
extern int		RmSetTaskArgument(RmTask, int, char *);
extern const char	*RmGetTaskArgument(RmTask, int);
extern int		RmCountTaskArguments(RmTask);

extern RmNetwork	RmNewNetwork(void);
extern int		RmFreeNetwork(RmNetwork);
extern const RmID	RmGetNetworkID(RmNetwork);
extern int		RmSetNetworkID(RmNetwork, RmID);
extern RmProcessor	RmAddTailProcessor(RmNetwork, RmProcessor);
extern RmProcessor	RmAddHeadProcessor(RmNetwork, RmProcessor);
extern RmProcessor	RmRemoveProcessor(RmProcessor);
extern RmProcessor	RmPreInsertProcessor(RmProcessor, RmProcessor);
extern RmProcessor	RmPostInsertProcessor(RmProcessor, RmProcessor);
extern RmProcessor	RmFirstProcessor(RmNetwork);
extern RmProcessor	RmLastProcessor(RmNetwork);
extern RmProcessor	RmNextProcessor(RmProcessor);
extern RmProcessor	RmPreviousProcessor(RmProcessor);
extern bool		RmEmptyNetwork(RmNetwork);
extern int		RmSizeofNetwork(RmNetwork);
extern int		RmCountProcessors(RmNetwork);
extern RmNetwork	RmParentNetwork(RmProcessor);
extern int RmApplyNetwork(RmNetwork, int (*fn)(RmProcessor, ...), ...);
extern int RmSearchNetwork(RmNetwork, int (*fn)(RmProcessor, ...), ...);
extern int		RmMergeNetworks(RmNetwork, RmNetwork);
extern int		RmMakeLink(RmProcessor, int, RmProcessor, int);
extern int		RmBreakLink(RmProcessor, int);
extern int		RmResolveLinks(RmNetwork);
extern int		RmCountLinks(RmProcessor);
extern RmProcessor	RmFollowLink(RmProcessor, int, int *);
extern int		RmGetLinksFlags(RmProcessor, int);
extern RmNetwork	RmRootNetwork(RmProcessor);
extern bool		RmIsNetwork(RmProcessor);
extern bool		RmIsProcessor(RmProcessor);

extern RmTaskforce	RmNewTaskforce(void);
extern int		RmFreeTaskforce(RmTaskforce);
extern const RmID	RmGetTaskforceID(RmTaskforce);
extern int		RmSetTaskforceID(RmTaskforce, RmID);
extern RmTask		RmAddTailTask(RmTaskforce, RmTask);
extern RmTask		RmAddHeadTask(RmTaskforce, RmTask);
extern RmTask		RmRemoveTask(RmTask);
extern RmTask		RmPreInsertTask(RmTask, RmTask);
extern RmTask		RmPostInsertTask(RmTask, RmTask);
extern RmTask		RmFirstTask(RmTaskforce);
extern RmTask		RmLastTask(RmTaskforce);
extern RmTask		RmNextTask(RmTask);
extern RmTask		RmPreviousTask(RmTask);
extern bool		RmEmptyTaskforce(RmTaskforce);
extern int		RmSizeofTaskforce(RmTaskforce);
extern int		RmCountTasks(RmTaskforce);
extern RmTaskforce	RmParentTaskforce(RmTask);
extern int 	RmApplyTaskforce(RmTaskforce, int (*fn)(RmTask, ...), ...);
extern int 	RmSearchTaskforce(RmTaskforce, int (*fn)(RmTask, ...), ...);
extern int		RmMakeChannel(RmTask, int, RmTask, int);
extern int		RmBreakChannel(RmTask, int);
extern int		RmResolveChannels(RmTaskforce);
extern int		RmCountChannels(RmTask);
extern RmTask		RmFollowChannel(RmTask, int, int *);
extern int		RmGetChannelFlags(RmTask, int);
extern int		RmConnectChannelToFile(RmTask, int, char *, int);
extern const char	*RmFollowChannelToFile(RmTask, int, int *);
extern RmTaskforce	RmRootTaskforce(RmTask);
extern bool		RmIsTaskforce(RmTask);
extern bool		RmIsTask(RmTask);

extern RmNetwork	RmGetWholeNetwork(int *);
extern int		RmLastChangeWholeNetwork(void);
extern RmNetwork	RmGetDomain(int *);
extern int		RmLastChangeDomain(void);
extern RmProcessor	RmObtainProcessor(RmProcessor, int *);
extern int		RmSetProcessorShareable(RmProcessor);
extern int		RmSetProcessorExclusive(RmProcessor);
extern bool		RmIsProcessorShareable(RmProcessor);
extern bool		RmIsProcessorExclusive(RmProcessor);
extern int		RmSetProcessorTemporary(RmProcessor);
extern int		RmSetProcessorPermanent(RmProcessor);
extern bool		RmIsProcessorTemporary(RmProcessor);
extern bool		RmIsProcessorPermanent(RmProcessor);
extern int		RmReleaseProcessor(RmProcessor);
extern RmNetwork	RmObtainNetwork(RmNetwork, bool exact, int *);
extern RmProcessor	RmFindMatchingProcessor(RmProcessor, RmNetwork);
extern int		RmReleaseNetwork(RmNetwork);
extern int		RmSetNetworkShareable(RmNetwork);
extern int		RmSetNetworkExclusive(RmNetwork);
extern int		RmSetNetworkTemporary(RmNetwork);
extern int		RmSetNetworkPermanent(RmNetwork);
extern int		RmReconfigureNetwork(RmNetwork);
extern int		RmTestConfiguration(RmNetwork);
extern int		RmInstallNetwork(RmNetwork);

extern int		RmAddHardwareFacility(RmNetwork, RmHardwareFacility *);
extern int		RmApplyHardwareFacilities(RmNetwork,
			int (*fn)(RmHardwareFacility *, ...), ...);
extern int		RmSearchHardwareFacilities(RmNetwork,
			int (*fn)(RmHardwareFacility *, ...), ...);

extern	int		RmRead(char *filename, RmNetwork *, RmTaskforce *);
extern	int		RmWrite(char *filename, RmNetwork, RmTaskforce);
extern	Object		*RmMapProcessorToObject(RmProcessor);
extern	RmProcessor	RmInsertProcessor(RmNetwork, RmProcessor);
extern	RmNetwork	RmGetNetworkHierarchy(int *);

/* Processor purposes */
#define	RmP_Helios	0x01
#define RmP_System	0x02
#define RmP_IO		0x03
#define RmP_Native	0x04
#define RmP_Mask	0x0F

/* Processor types */
#define RmT_Unknown		 0
#define RmT_Default		 1
#define RmT_Known		11
#define RmT_T800		 2
#define RmT_T414		 3 
#define RmT_T425		 4
#define RmT_T400		 5
#define RmT_T212		 6
#define RmT_H1			 7
#define RmT_i860		 8
#define RmT_Arm		 	 9
#define RmT_680x0		10
extern  char *RmT_Names[RmT_Known];

/* Processor states */
#define RmS_ShouldBeReset	0x00010
#define RmS_Reset		0x00020
#define RmS_Booting		0x00040
#define	RmS_AutoBoot		0x00080
#define RmS_Running		0x00100
#define RmS_Suspicious		0x00200
#define	RmS_Crashed		0x00400
#define RmS_Dead		0x00800
#define RmS_Special		0x01000

/* RmLib error codes */
#define RmE_Success		    0
#define RmE_Error		0x100	/* smallest error code */
#define RmE_NotProcessor	0x100	/* processor argument is invalid */
#define	RmE_NotTask		0x100   /* ... */
#define RmE_NotNetwork		0x101
#define	RmE_NotTaskforce	0x102
#define RmE_WrongNetwork	0x103	/* Attempting to make invalid */
#define RmE_WrongTaskforce	0x104	/* connection */
#define RmE_InUse		0x105	/* processor already part of network */
#define RmE_Corruption		0x106	/* system memory corruption detected */
#define RmE_ReadOnly		0x107	/* write-operation illegal */
#define RmE_BadArgument		0x108	/* e.g. invalid processor type */
#define RmE_NoMemory		0x109	/* Malloc() failure */
#define RmE_NotFound		0x10A	/* a search failed, e.g. IsAttr */
#define RmE_TooLong		0x10B	/* a string argument is too long */
#define RmE_NotRootNetwork	0x10C
#define RmE_NoAccess		0x10D
#define RmE_OldStyle		0x10E
#define RmE_BadFile		0x10F
#define RmE_CommsBreakdown	0x110
#define RmE_Skip		0x111
#define RmE_Native		0x112	/* operation is inappropriate */
#define	RmE_MissingServer	0x113
#define	RmE_PartialSuccess	0x114

/* Miscellaneous constants */
#define RmM_ExternalProcessor	((RmProcessor) 1)
#define RmM_ExternalTask	((RmTask) 1)
#define RmM_NoProcessor		((RmProcessor) NULL)
#define RmM_NoTask		((RmTask) NULL)
#define RmM_AnyLink		(-1)
#define RmM_AnyChannel		(-1)
#define RmM_NoOwner		0

/* Flags */
#define RmF_Configurable	0x01000000
#define RmF_AnyLink		0x02000000
#define RmF_AnyChannel		0x01000000
#define RmF_ToFile		0x02000000

/* Hardware facilities */
#define RmH_ResetDriver		0x01
#define RmH_ResetCommand	0x02
#define RmH_ConfigureDriver	0x03

/* Open Modes */
#define RmO_ReadOnly	1
#define RmO_WriteOnly	2
#define RmO_ReadWrite	3
#define RmO_Create	0x0100
#define RmO_Exclusive	0x0200
#define RmO_Truncate	0x0400
#define RmO_Append	0x1000

/**
*** Link modes
**/
#define	RmL_NotConnected	0
#define	RmL_Dumb		1
#define RmL_Intelligent		2
#define RmL_Pending		3

#endif   /* __rmlib_h */


