/*
 * $Header: CoreP.h,v 1.5 88/02/26 12:36:07 swick Exp $
 */

/***********************************************************
Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

#ifndef XtCoreP_h
#define XtCoreP_h

#ifndef _XtintrinsicP_h
#include <X11/IntrinsP.h>
#endif

#define UnspecifiedPixmap ((Pixmap)2)

#ifndef __tmprivate_h

typedef struct _ActionsRec *	ActionPtr;

typedef struct _ActionsRec
{
    char * 			token;		/* string token for procedure name */
    unsigned int 		index;         	/* index into quarkTable to find proc */
    String *			params;		/* pointer to array of params */
    unsigned long 		num_params;	/* number of params */
    ActionPtr 			next;		/* next action to perform */
} ActionRec;

typedef struct _StateRec
{
    int 			index;		/* index of event into EventObj table */
    ActionPtr 			actions;	/* rhs   list of actions to perform */
    StatePtr 			nextLevel;	/* the next level points to the next event in one event sequence */
    StatePtr 			next;		/* points to other event state at same level */
    StatePtr 			forw;		/* points to next state in list of all states */
    Boolean 			cycle;		/* true iff nextLevel is a loop */
}  StateRec;

#endif

#ifndef STATE_PTR_DEFINED

typedef struct _StateRec *	StatePtr;
#define STATE_PTR_DEFINED

#endif

typedef struct _TMRec
{
    XtTranslations			translations; 		/* private to Translation Manager    */
    XtActionProc *			proc_table;     	/* procedure bindings for actions    */
    StatePtr				current_state;  	/* Translation Manager state ptr     */
    unsigned long			lastEventTime;
} TMRec;


#ifndef _Event_h_

typedef struct _XtEventRec *	_XtEventTable;

typedef struct _XtEventRec
{
     _XtEventTable	next;
     EventMask		mask;
     Boolean		non_filter;
     Boolean		select;
     Boolean		raw;
     XtEventHandler	proc;
     caddr_t		closure;
}XtEventRec;

#endif

/***************************************************************
 * Widget Core Data Structures
 *
 *
 **************************************************************/

typedef struct _CorePart
{
    Widget	    			self;			/* pointer to widget itself	     */
    WidgetClass	    			widget_class;		/* pointer to Widget's ClassRec	     */
    Widget	    			parent;			/* parent widget	  	     */
    String          			name;			/* widget resource name		     */
    XrmName         			xrm_name;		/* widget resource name quarkified   */
    Screen *	    			screen;			/* window's screen		     */
    Colormap        			colormap;       	/* colormap                          */
    Window	    			window;			/* window ID			     */
    Position        			x, y;			/* window position		     */
    Dimension       			width, height;		/* window dimensions		     */
    Cardinal        			depth;			/* number of planes in window        */
    Dimension       			border_width;		/* window border width		     */
    Pixel	    			border_pixel;		/* window border pixel		     */
    Pixmap          			border_pixmap;		/* window border pixmap or NULL      */
    Pixel	    			background_pixel;	/* window background pixel	     */
    Pixmap          			background_pixmap;	/* window background pixmap or NULL  */
    struct _XtEventRec *		event_table;		/* private to event dispatcher       */
    struct _TMRec   			tm;             	/* translation management            */
    caddr_t         			constraints;    	/* constraint record                 */
    Boolean         			visible;		/* is window mapped and not occluded?*/
    Boolean	    			sensitive;		/* is widget sensitive to user events*/
    Boolean        			ancestor_sensitive;	/* are all ancestors sensitive?      */
    Boolean        			managed;		/* is widget geometry managed?	     */
    Boolean	   			mapped_when_managed;	/* map window if it's managed?       */
    Boolean         			being_destroyed;	/* marked for destroy		     */
    XtCallbackList  			destroy_callbacks;	/* who to call when widget destroyed */
    WidgetList      			popup_list;       	/* list of popups                    */
    Cardinal        			num_popups;       	/* how many popups                   */
} CorePart;

typedef struct _WidgetRec
{
    CorePart    	core;
} WidgetRec;


/* 
 * Because of compiler problem with static initialisers
 * we must define WidgetRec ofsets
 */

#define W_offset_self			0
#define W_offset_widget_class   	W_offset_self 			+ sizeof(Widget)
#define W_offset_parent    		W_offset_widget_class 		+ sizeof(WidgetClass)
#define W_offset_name    		W_offset_parent 		+ sizeof(Widget)
#define W_offset_xrm_name    		W_offset_name 			+ sizeof(String)
#define W_offset_screen    		W_offset_xrm_name 		+ sizeof(XrmName)
#define W_offset_colormap    		W_offset_screen			+ sizeof(Screen *)
#define W_offset_window    		W_offset_colormap 		+ sizeof(Colormap)
#define W_offset_x    			W_offset_window 		+ sizeof(Window)
#define W_offset_y    			W_offset_x 			+ sizeof(Position)
#define W_offset_width    		W_offset_y 			+ sizeof(Position)
#define W_offset_height    		W_offset_width 			+ sizeof(Dimension)
#define W_offset_depth    		W_offset_height 		+ sizeof(Dimension)
#define W_offset_border_width   	W_offset_depth 			+ sizeof(Cardinal)
#define W_offset_border_pixel   	W_offset_border_width 		+ sizeof(Dimension)
#define W_offset_border_pixmap  	W_offset_border_pixel 		+ sizeof(Pixel)
#define W_offset_background_pixel  	W_offset_border_pixmap 		+ sizeof(Pixmap) 
#define W_offset_background_pixmap 	W_offset_background_pixel 	+ sizeof(Pixel)  
#define W_offset_event_table    	W_offset_background_pixmap	+ sizeof(Pixmap)
#define W_offset_tm    			W_offset_event_table 		+ sizeof(struct _XtEventRec *)
#define W_offset_constraints    	W_offset_tm 			+ sizeof(struct _TMRec)
#define W_offset_visible    		W_offset_constraints 		+ sizeof(caddr_t)
#define W_offset_sensitive    		W_offset_visible 		+ sizeof(Boolean)
#define W_offset_ancestor_sensitive  	W_offset_sensitive 		+ sizeof(Boolean)
#define W_offset_managed    		W_offset_ancestor_sensitive 	+ sizeof(Boolean)
#define W_offset_mapped_when_managed   	W_offset_managed 		+ sizeof(Boolean)
#define W_offset_being_destroyed  	W_offset_mapped_when_managed 	+ sizeof(Boolean)
/* NB/ the next field is word aligned */
#define W_offset_destroy_callbacks  	((W_offset_being_destroyed 	+ sizeof(Boolean) + 3 ) & ~3)
#define W_offset_popup_list    		W_offset_destroy_callbacks 	+ sizeof(XtCallbackList)
#define W_offset_num_popups    		W_offset_popup_list 		+ sizeof(_WidgetList)

#define W_offset_tm_translations	W_offset_tm


#ifndef _XtintrinsicI_h

typedef struct _XtOffsetRec
{
     struct _XtOffsetRec *	next;
     XrmQuark       		name;
     int            		offset;
} XtOffsetRec, *_XtOffsetList;

#endif

/******************************************************************
 *
 * Core Class Structure. Widgets, regardless of their class, will have
 * these fields.  All widgets of a given class will have the same values
 * for these fields.  Widgets of a given class may also have additional
 * common fields.  These additional fields are included in incremental
 * class structures, such as CommandClass.
 *
 * The fields that are specific to this subclass, as opposed to fields that
 * are part of the superclass, are called "subclass fields" below.  Many
 * procedures are responsible only for the subclass fields, and not for
 * any superclass fields.
 *
 ********************************************************************/

typedef struct _CoreClassPart
{
    WidgetClass     			superclass;		/* pointer to superclass ClassRec   */
    String          			class_name;		/* widget resource class name       */
    Cardinal        			widget_size;		/* size in bytes of widget record   */
    XtProc	    			class_initialize;   	/* class initialization proc	    */
    XtWidgetClassProc 			class_part_initialize;	/* dynamic initialization           */
    Boolean         			class_inited;       	/* has class been initialized?      */
    XtInitProc      			initialize;		/* initialize subclass fields       */
    XtArgsProc      			initialize_hook;    	/* notify that initialize called    */
    XtRealizeProc   			realize;		/* XCreateWindow for widget	    */
    XtActionList    			actions;		/* widget semantics name to proc map*/
    Cardinal	    			num_actions;		/* number of entries in actions     */
    XtResourceList  			resources;		/* resources for subclass fields    */
    Cardinal        			num_resources;      	/* number of entries in resources   */
    XrmClass        			xrm_class;		/* resource class quarkified	    */
    Boolean         			compress_motion;    	/* compress MotionNotify for widget */
    Boolean         			compress_exposure;  	/* compress Expose events for widget*/
    Boolean         			compress_enterleave;	/* compress enter and leave events  */
    Boolean         			visible_interest;   	/* select for VisibilityNotify      */
    XtWidgetProc    			destroy;		/* free data for subclass pointers  */
    XtWidgetProc    			resize;			/* geom manager changed widget size */
    XtExposeProc    			expose;			/* rediplay window		    */
    XtSetValuesFunc 			set_values;		/* set subclass resource values     */
    XtArgsFunc      			set_values_hook;    	/* notify that set_values called    */
    XtAlmostProc    			set_values_almost;  	/* set_values got "Almost" geo reply*/
    XtArgsProc      			get_values_hook;    	/* notify that get_values called    */
    XtWidgetProc    			accept_focus;       	/* assign input focus to widget     */
    XtVersionType   			version;	        /* version of intrinsics used	    */
    struct _XtOffsetRec *		callback_private;	/* list of callback offsets         */
    String         			tm_table;            	/* state machine                    */
    XtGeometryHandler 			query_geometry;		/* return preferred geometry        */
} CoreClassPart;

typedef struct _WidgetClassRec
{
    CoreClassPart core_class;
} WidgetClassRec;

globalref WidgetClassRec		widgetClassRec;

extern int 				_XtInheritTranslations;

#define XtInheritTranslations  		((String)           &_XtInheritTranslations)
#define XtInheritRealize 		((XtRealizeProc)     _XtInheritRealize)
#define XtInheritResize 		((XtWidgetProc)      _XtInheritResize)
#define XtInheritExpose 		((XtExposeProc)      _XtInheritExpose)
#define XtInheritSetValuesAlmost 	((XtAlmostProc)      _XtInheritSetValuesAlmost)
#define XtInheritAcceptFocus 		((XtWidgetProc)      _XtInheritAcceptFocus)
#define XtInheritQueryGeometry 		((XtGeometryHandler) _XtInheritQueryGeometry)

#endif /* _XtCoreP_h */

/* DON'T ADD STUFF AFTER THIS #endif */
