/*
 * $Header: Intrinsic.h,v 1.76 88/02/26 12:40:57 swick Exp $
 */

/***********************************************************
Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

/*
 * Header: copyright.h v1.0 15th December 1988
 *
 *
 * Copyright (c) 1988, 1989 by Perihelion Software Ltd.
 * 
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting
 * documentation, and that the name of Perihelion Software Ltd.
 * not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written
 * prior permission.  Perihelion Software Ltd. makes no
 * representations about the suitability of this software
 * for any purpose.  It is provided "as is" without
 * express or implied warranty.
 *
 * Perihelion  Software  Ltd  disclaims all  warranties with
 * regard to this software, including all implied warranties
 * of merchantability  and fitness for a particular purpose.
 * In no event shall Perihelion Software Ltd be  liable  for 
 * any  special,  indirect or  consequential damages  or any
 * damages  whatsoever resulting from loss of use,  data  or
 * profits,  whether in an action of contract, negligence or
 * other  tortious action,  arising out of  or in connection 
 * with the use or performance of this software.
 */

#ifndef _XtIntrinsic_h
#define _XtIntrinsic_h

#include <X11/Xlib.h>
#include <X11/Xresource.h>

#ifdef helios
#include <xtypes.h>
#else
#include <sys/types.h>
#endif

#include <syslib.h>

#ifndef NULL
define NULL 	0
#endif

#define globalref extern
#define globaldef

#ifndef FALSE
#define FALSE 	0
#define TRUE  	1
#endif

#define XtNumber( arr )			((Cardinal) (sizeof(arr) / sizeof(arr[ 0 ])))

#ifdef helios

#ifndef __stddef_h
#include <stddef.h>
#endif

#define XtOffset( type, field )		((char *)&(((type *)0)->field) - (char *)0)

#else	

#define XtOffset( type, field )    	((unsigned int)&(((type)NULL)->field))

#endif


#ifndef __tmprivate_h

typedef struct _EventRec
{
    unsigned long 		modifiers;
    unsigned long 		modifierMask;
    unsigned long 		eventType;
    unsigned long 		eventCode;
    unsigned long 		eventCodeMask;
} Event;

#ifndef STATE_PTR_DEFINED

typedef struct _StateRec *	StatePtr;
#define STATE_PTR_DEFINED

#endif

typedef struct _EventObjRec *	EventObjPtr;

typedef struct _EventObjRec
{
    Event 			event;		/* X event description */
    StatePtr 			state;		/* pointer to linked lists of state info */
} EventObjRec;

typedef struct _TranslationData
{
    unsigned int		numEvents;
    unsigned int		eventTblSize;
    EventObjPtr			eventObjTbl;
    unsigned long		clickTime;
    unsigned long		lastEventTime;
    unsigned int        	numQuarks;   	/* # of entries in quarkTable */
    unsigned int        	quarkTblSize; 	/*total size of quarkTable */
    XrmQuark *           	quarkTable;  	/* table of quarkified rhs*/
    StatePtr			head;		/* head of list of all states */
} TranslationData;

#endif /* __tmprivate_h */

typedef char *				String;
typedef struct _WidgetRec *		Widget;
typedef Widget *			WidgetList;
typedef struct _WidgetClassRec *	WidgetClass;
typedef struct _CompositeRec *		CompositeWidget;
typedef struct _XtActionsRec *		XtActionList;
typedef struct _XtResource *		XtResourceList;
typedef unsigned int   			Cardinal;
typedef char				Boolean;
typedef unsigned long	*		Opaque;
typedef struct _TranslationData	*	XtTranslations;
typedef struct _XtCallbackRec *    	XtCallbackList;
typedef unsigned long   		XtValueMask;
typedef unsigned long   		XtIntervalId;
typedef unsigned long   		XtInputId;
typedef unsigned int    		XtGeometryMask;
typedef unsigned long   		XtGCMask;   	/* Mask of values that are used by widget */
typedef unsigned long   		Pixel;	    	/* Index into colormap	        	  */
typedef int				Position;   	/* Offset from 0 coordinate	          */
typedef unsigned int			Dimension;  	/* Size in pixels		          */


/****************************************************************
 *
 * System Dependent Definitions
 *
 *
 * XtArgVal ought to be a union of caddr_t, char *, long, int *, and proc *
 * but casting to union types is not really supported.
 *
 * So the typedef for XtArgVal should be chosen such that
 *
 *      sizeof (XtArgVal) >=	sizeof(caddr_t)
 *				sizeof(char *)
 *				sizeof(long)
 *				sizeof(int *)
 *				sizeof(proc *)
 *
 * ArgLists rely heavily on the above typedef.
 *
 ****************************************************************/
typedef long XtArgVal;

typedef struct
{
    String	name;
    XtArgVal	value;
} Arg, *ArgList;

extern Boolean			XtIsSubclass( Widget, WidgetClass );

#define XtIsComposite( widget )	XtIsSubclass( (widget), compositeWidgetClass )
#define XtMapWidget( widget )	XMapWindow(   XtDisplay( widget ), XtWindow( widget ) )
#define XtUnmapWidget( widget )	XUnmapWindow( XtDisplay( widget ), XtWindow( widget ) )


extern Widget			XtCreateWidget(           String name, WidgetClass widget_class, Widget parent, ArgList args, Cardinal num_args );

extern Widget 			XtCreateManagedWidget(    String name, WidgetClass widget_class, Widget parent, ArgList args, Cardinal num_args );

extern Widget 			XtCreateApplicationShell( String name, WidgetClass widget_class,                ArgList args, Cardinal num_args );

extern void 			XtRealizeWidget( Widget widget );

extern Boolean 			XtIsRealized( Widget widget );

extern void 			XtDestroyWidget( Widget widget );

extern void 			XtSetSensitive( Widget widget, Boolean sensitive );

extern void 			XtSetMappedWhenManaged( Widget widget, Boolean mappedWhenManaged );

extern Widget 			XtNameToWidget( Widget root , String name );

extern Widget 			XtWindowToWidget( Display *display, Window window );


/**********************************************************
 *
 * Composite widget Procedures
 *
 **********************************************************/

extern void 			XtManageChildren(   WidgetList children, Cardinal num_children );

extern void 			XtUnmanageChildren( WidgetList children , Cardinal num_children );

extern void 			XtManageChild(   Widget child );

extern void 			XtUnmanageChild( Widget child );

/*************************************************************
 *
 *  Callbacks
 *
 **************************************************************/

typedef void 			(*XtCallbackProc)( Widget widget, caddr_t closure, caddr_t calldata );

/* closure:-  data the application registered */
/* callData:- widget instance specific data passed to application */

typedef struct _XtCallbackRec
{
    XtCallbackProc  callback;
    caddr_t	    closure;
} XtCallbackRec;

extern void 			XtAddCallback(        Widget widget, String callback_name, XtCallbackProc callback, caddr_t closure );

extern void 			XtRemoveCallback(     Widget widget, String callback_name, XtCallbackProc callback, caddr_t closure );

extern void 			XtAddCallbacks(       Widget widget, String callback_name, XtCallbackList callbacks );

extern void 			XtRemoveCallbacks(    Widget widget, String callback_name, XtCallbackList callbacks );

extern void 			XtRemoveAllCallbacks( Widget widget, String callback_name );
	
extern void 			XtCallCallbacks(      Widget widget, String callback_name, caddr_t callData );

/* These utilities are here on Leo's request. We should think about them */
/* and decide if they really belong in the intrinsics, or a level above */

/* In particular, this procedure is nasty... */

extern void 			XtOverrideCallback( Widget widget, String callback_name, XtCallbackProc callback );

typedef enum
{
	XtCallbackNoList,
	XtCallbackHasNone,
	XtCallbackHasSome
} XtCallbackStatus;

extern XtCallbackStatus		XtHasCallbacks( Widget widget, String callback_name );


/****************************************************************
 *
 * Toolkit initialization
 *
 ****************************************************************/

extern Widget 			XtInitialize( String name, String class, XrmOptionDescRec *options, Cardinal num_options, Cardinal * argc, /* returns count of args not processed */ char ** argv );


/****************************************************************
 *
 * Memory Management
 *
 ****************************************************************/

#define XtNew(type) 		((type *) XtMalloc((unsigned) sizeof(type)))

extern char *			XtMalloc( Cardinal size );

extern char *			XtCalloc( Cardinal num, Cardinal size );

extern char *			XtRealloc( char * ptr, Cardinal num );

extern void 			XtFree( char * ptr );


/****************************************************************
 *
 * Arg lists
 *
 ****************************************************************/

#define XtSetArg(arg, n, d) 	( (arg).name = (n), (arg).value = (XtArgVal)(d) )

extern ArgList 			XtMergeArgLists( ArgList args1, Cardinal num_args1, ArgList args2, Cardinal num_args2 );


/****************************************************************
 *
 * Event Management
 *
 ****************************************************************/

typedef void 			(*XtEventHandler)( Widget widget, caddr_t closure, XEvent * event );

typedef unsigned long 		EventMask;

#define XtAllEvents 		((EventMask) -1L)


extern void 			XtAddEventHandler(       Widget widget, EventMask eventMask, Boolean nonmaskable, XtEventHandler proc, caddr_t closure );

extern void 			XtRemoveEventHandler(    Widget widget, EventMask eventMask, Boolean nonmaskable, XtEventHandler proc, caddr_t closure );

extern void 			XtAddRawEventHandler(    Widget widget, EventMask eventMask, Boolean nonmaskable, XtEventHandler proc, caddr_t closure );

extern void 			XtRemoveRawEventHandler( Widget	widget, EventMask eventMask, Boolean nonmaskable, XtEventHandler proc, caddr_t closure );

extern void 			XtAddGrab( 		 Widget widget, Boolean exclusive, Boolean spring_loaded );

extern void 			XtRemoveGrab( 		 Widget widget );

extern void 			XtDispatchEvent( XEvent * event );

extern void 			XtMainLoop( void );


/****************************************************************
 *
 * Event Gathering Routines
 *
 ****************************************************************/

typedef unsigned long		XtInputMask;

#define XtInputNoneMask		0L
#define XtInputReadMask		(1L<<0)
#define XtInputWriteMask	(1L<<1)
#define XtInputExceptMask	(1L<<2)

extern Atom 			XtHasInput;
extern Atom 			XtTimerExpired;


typedef void 			(*XtTimerCallbackProc)( caddr_t closure, XtIntervalId * id );


extern XtIntervalId 		XtAddTimeOut( unsigned long interval, XtTimerCallbackProc proc, caddr_t closure );

extern void 			XtRemoveTimeOut( XtIntervalId timer );


typedef void 			(* XtInputCallbackProc)( caddr_t closure, int * source, XtInputId * id );


extern XtInputId 		XtAddInput( Stream * source, caddr_t condition, XtInputCallbackProc proc, caddr_t closure );

extern void 			XtRemoveInput( XtInputId id );

extern void 			XtNextEvent( XEvent * event );

extern int 			XtPeekEvent( XEvent * event );

extern Boolean 			XtPending( void );


/****************************************************************
 *
 * Geometry Management
 *
 ****************************************************************/

#define XtSMDontChange	5 /* don't change the stacking order stack_mode */

typedef struct
{
    XtGeometryMask 	request_mode;
    Position 		x, y;
    Dimension 		width, height, border_width;
    Widget 		sibling;
    int 		stack_mode;	/* Above, Below, TopIf, BottomIf, Opposite */
} XtWidgetGeometry;

typedef enum 
{
    XtGeometryYes,        /* Request accepted. */
    XtGeometryNo,         /* Request denied. */
    XtGeometryAlmost,     /* Request denied, but willing to take replyBox. */
    XtGeometryDone	  /* Request accepted and done. */
} XtGeometryResult;


extern XtGeometryResult		XtMakeGeometryRequest( Widget widget, XtWidgetGeometry * request, XtWidgetGeometry * reply );

extern XtGeometryResult 	XtMakeResizeRequest( Widget widget, Dimension width, Dimension height, Dimension * replyWidth, Dimension * replyHeight );

extern void 			XtTransformCoords( Widget w, Position x, Position y, Position * rootx, Position * rooty );

extern XtGeometryResult 	XtQueryGeometry( Widget	widget, XtWidgetGeometry * intended, XtWidgetGeometry * reply_return );


/****************************************************************
 *
 * Widget resource initialization, reading, writing
 *
 ****************************************************************/

typedef struct _XtResource
{
    String	resource_name;	/* Resource name			    */
    String	resource_class;	/* Resource class			    */
    String	resource_type;	/* Representation type desired		    */
    Cardinal	resource_size;	/* Size in bytes of representation	    */
    Cardinal	resource_offset;/* Offset from base to put resource value   */
    String	default_type;	/* representation type of specified default */
    caddr_t	default_addr;   /* Address of default resource		    */
} XtResource;

extern void 			XtGetSubresources( Widget widget, caddr_t base, String name, String class, XtResourceList resources, Cardinal num_resources, ArgList args, Cardinal num_args );

extern void 			XtSetValues( Widget widget, ArgList args, Cardinal num_args );

extern void 			XtGetValues( Widget widget, ArgList args, Cardinal num_args );

extern void 			XtSetSubvalues( caddr_t base, XtResourceList resources, Cardinal num_resources, ArgList	args, Cardinal num_args );

extern void 			XtGetSubvalues( caddr_t base, XtResourceList resources, Cardinal num_resources, ArgList	args, Cardinal num_args );


/****************************************************************
 *
 * Translation Management
 *
 ****************************************************************/


typedef void (*			XtActionProc)( Widget widget, XEvent * event, String * params, Cardinal * num_params );

typedef struct _XtActionsRec
{
    char *    		string;
    XtActionProc 	proc;
} XtActionsRec;

extern XtTranslations 		XtParseTranslationTable( String source );

extern void 			XtFreeTranslations( XtTranslations stateTable );

extern void 			XtOverrideTranslations( Widget widget, XtTranslations new );

extern void 			XtAugmentTranslations( Widget widget, XtTranslations new );

extern void 			XtAddActions( XtActionList action, Cardinal num_actions );


/*************************************************************
 *
 * Error Handling
 *
 ************************************************************/

extern void 			XtError(   String message );

extern void 			XtWarning( String message );

extern void 			XtSetErrorHandler(   void (*errorProc)(String) );

extern void			XtSetWarningHandler( void (*errorProc)(String) );


/*************************************************************
 *
 * popup Handling
 *
 ************************************************************/

extern Widget 			XtCreatePopupShell( String name, WidgetClass widgetClass, Widget parent, ArgList args, Cardinal num_args );

typedef enum
{
	XtGrabNone,
	XtGrabNonexclusive,
	XtGrabExclusive
} XtGrabKind;


extern void 			XtPopup( Widget widget, XtGrabKind grab_kind );

extern void 			XtCallbackNone(         Widget widget, caddr_t closure, caddr_t call_data );

extern void 			XtCallbackNonexclusive( Widget widget, caddr_t closure, caddr_t call_data );

extern void 			XtCallbackExclusive(    Widget widget, caddr_t closure, caddr_t call_data );

extern void 			XtPopdown( Widget );

typedef struct
{
    Widget  shell_widget;
    Widget  enable_widget;
} XtPopdownIDRec, *XtPopdownID;
    
extern void 			XtCallbackPopdown( 	Widget widget, caddr_t closure, caddr_t call_data );


/****************************************************************
 *
 * Graphic Context Management
 *****************************************************************/

extern GC 			XtGetGC( Widget widget, XtGCMask valueMask, XGCValues * values );

extern void 			XtDestroyGC( GC gc );


/*************************************************************
 *
 * Misc
 *
 ************************************************************/

extern Cursor 			XtGetCursor( Display * dpy, int num );


/*************************************************************
 *
 * Information routines
 *
 ************************************************************/

#ifndef _XtIntrinsicP_h

/* We're not included from the private file, so define these */

extern Display *		XtDisplay(    Widget widget );

extern Screen *			XtScreen(     Widget widget );

extern Window 			XtWindow(     Widget widget );

extern WidgetClass 		XtSuperclass( Widget widget );

extern WidgetClass 		XtClass(      Widget widget );

extern Boolean 			XtIsManaged(  Widget widget );

extern Boolean 			XtIsRealized( Widget widget );

extern Widget 			XtParent(     Widget widget );

#endif /* _XtIntrinsicP_h */

/*
 * function definitions added by NC
 */
 
extern void	XtGetApplicationResources( Widget, caddr_t, XtResourceList, Cardinal, ArgList, Cardinal );

#ifndef _XtComposite_h
#include <X11/Composite.h>
#endif

#endif /* _XtIntrinsic_h */

/* DON'T ADD STUFF AFTER THIS #endif */
