/* $Header: Xutil.h,v 11.41 88/02/14 11:55:28 rws Exp $ */

/***********************************************************
Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

/*
 * Copyright (c) 1988, 1989 by Perihelion Software Ltd.
 * 
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies and that both that copyright
 * notice and this permission notice appear in supporting
 * documentation, and that the name of Perihelion Software Ltd.
 * not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written
 * prior permission.  Perihelion Software Ltd. makes no
 * representations about the suitability of this software
 * for any purpose.  It is provided "as is" without
 * express or implied warranty.
 *
 * Perihelion  Software  Ltd  disclaims all  warranties with
 * regard to this software, including all implied warranties
 * of merchantability  and fitness for a particular purpose.
 * In no event shall Perihelion Software Ltd be  liable  for 
 * any  special,  indirect or  consequential damages  or any
 * damages  whatsoever resulting from loss of use,  data  or
 * profits,  whether in an action of contract, negligence or
 * other  tortious action,  arising out of  or in connection 
 * with the use or performance of this software.
 */

#ifndef _XUTIL_H_
#define _XUTIL_H_

#ifndef _XLIB_H_
#include <X11/Xlib.h>
#endif

/* 
 * Bitmask returned by XParseGeometry().  Each bit tells if the corresponding
 * value (x, y, width, height) was found in the parsed string.
 */

#define NoValue		0x0000
#define XValue  	0x0001
#define YValue		0x0002
#define WidthValue  	0x0004
#define HeightValue  	0x0008
#define AllValues 	0x000F
#define XNegative 	0x0010
#define YNegative 	0x0020

typedef struct
{
	long 		flags;		/* marks which fields in this structure are defined */
	int 		x;
	int		y;
	unsigned int 	width;
	unsigned int	height;
	int 		min_width;
	int		min_height;
	int 		max_width;
	int		max_height;
    	int 		width_inc;
    	int		height_inc;
	struct
	{
		int 	x;	/* numerator */
		int 	y;	/* denominator */
	}
			min_aspect,
			max_aspect;
} XSizeHints;

/*
 * The next block of definitions are for window manager properties that
 * clients and applications use for communication.
 */

/* flags argument in size hints */

#define USPosition	(1L << 0) /* user specified x, y */
#define USSize		(1L << 1) /* user specified width, height */

#define PPosition	(1L << 2) /* program specified position */
#define PSize		(1L << 3) /* program specified size */
#define PMinSize	(1L << 4) /* program specified minimum size */
#define PMaxSize	(1L << 5) /* program specified maximum size */
#define PResizeInc	(1L << 6) /* program specified resize increments */
#define PAspect		(1L << 7) /* program specified min and max aspect ratios */

#define PAllHints 	(PPosition | PSize | PMinSize | PMaxSize | PResizeInc | PAspect)

typedef struct
{
	long 	flags;		/* marks which fields in this structure are defined */
	Bool 	input;		/* does this application rely on the window manager to get keyboard input? */
	int 	initial_state;	/* see below */
	Pixmap 	icon_pixmap;	/* pixmap to be used as icon */
	Window 	icon_window; 	/* window to be used as icon */
	int 	icon_x;		/* initial position of icon */
	int	icon_y;
	Pixmap 	icon_mask;	/* icon mask bitmap */
	XID 	window_group;	/* id of related window group */
				/* this structure may be extended in the future */
} XWMHints;

/* definition for flags of XWMHints */

#define InputHint 		(1L << 0)
#define StateHint 		(1L << 1)
#define IconPixmapHint		(1L << 2)
#define IconWindowHint		(1L << 3)
#define IconPositionHint 	(1L << 4)
#define IconMaskHint		(1L << 5)
#define WindowGroupHint		(1L << 6)
#define AllHints 		(InputHint | StateHint | IconPixmapHint | IconWindowHint | IconPositionHint | IconMaskHint | WindowGroupHint)

/* definitions for initial window state */

#define DontCareState 	0	/* don't know or care */
#define NormalState 	1	/* most applications want to start this way */
#define ZoomState 	2	/* application wants to start zoomed */
#define IconicState 	3	/* application wants to start as an icon */
#define InactiveState 	4	/* application believes it is seldom used; some
    				   wm's may put it on inactive menu */
 
typedef struct
{
	int 	min_width;
	int	min_height;
	int 	max_width;
	int	max_height;
	int 	width_inc;
	int	height_inc;
} XIconSize;

typedef struct
{
	char *	res_name;
	char *	res_class;
} XClassHint;

/*
 * These macros are used to give some sugar to the image routines so that
 * naive people are more comfortable with them.
 */

#define XDestroyImage(ximage) 			((*((ximage)->f.destroy_image))((ximage)))
#define XGetPixel(ximage, x, y) 		((*((ximage)->f.get_pixel))((ximage), (x), (y)))
#define XPutPixel(ximage, x, y, pixel)		((*((ximage)->f.put_pixel))((ximage), (x), (y), (pixel)))
#define XSubImage(ximage, x, y, width, height)	((*((ximage)->f.sub_image))((ximage), (x), (y), (width), (height)))
#define XAddPixel(ximage, value)		((*((ximage)->f.add_pixel))((ximage), (value)))

/*
 * Compose sequence status structure, used in calling XLookupString.
 */

typedef struct _XComposeStatus
{
    char *	compose_ptr;		/* state table pointer */
    int 	chars_matched;		/* match state */
} XComposeStatus;

/*
 * Keysym macros, used on Keysyms to test for classes of symbols
 */

#define IsKeypadKey(keysym) 	(((unsigned)(keysym) >= XK_KP_Space) && ((unsigned)(keysym) <= XK_KP_Equal))

#define IsCursorKey(keysym) 	(((unsigned)(keysym) >= XK_Home)     && ((unsigned)(keysym) <  XK_Select))

#define IsPFKey(keysym) 	(((unsigned)(keysym) >= XK_KP_F1)     && ((unsigned)(keysym) <= XK_KP_F4))

#define IsFunctionKey(keysym) 	(((unsigned)(keysym) >= XK_F1)       && ((unsigned)(keysym) <= XK_F35))

#define IsMiscFunctionKey(keysym) 	(((unsigned)(keysym) >= XK_Select)   && ((unsigned)(keysym) <  XK_KP_Space))

#define IsModifierKey(keysym) 		(((unsigned)(keysym) >= XK_Shift_L)  && ((unsigned)(keysym) <= XK_Hyper_R))

/*
 * opaque reference to Region data type 
 */

#ifndef _REGION_H_

typedef struct
{
    	int 	x1;
	int	x2;
    	int	y1;
    	int	y2;
} Box, BOX, BoxRec, *BoxPtr;

typedef struct _XRegion
{
    int 	size;
    int 	numRects;
    BOX *	rects;
    BOX 	extents;
} REGION;

#endif /* !_REGION_H_ */

typedef struct _XRegion * Region; 

/* Return values from XRectInRegion() */
 
#define RectangleOut 	0
#define RectangleIn  	1
#define RectanglePart	2
 

/*
 * Information used by the visual utility routines to find desired visual
 * type from the many visuals a display may support.
 */

typedef struct
{
	Visual *	visual;
  	VisualID 	visualid;
  	int 		screen;
  	int 		depth;
  	int 		class;
  	unsigned long 	red_mask;
  	unsigned long 	green_mask;
  	unsigned long 	blue_mask;
  	int 		colormap_size;
  	int 		bits_per_rgb;
} XVisualInfo;

#define VisualNoMask		(0)
#define VisualIDMask 		(1 << 0)
#define VisualScreenMask	(1 << 1)
#define VisualDepthMask		(1 << 2)
#define VisualClassMask		(1 << 3)
#define VisualRedMaskMask	(1 << 4)
#define VisualGreenMaskMask	(1 << 5)
#define VisualBlueMaskMask	(1 << 6)
#define VisualColormapSizeMask	(1 << 7)
#define VisualBitsPerRGBMask	(1 << 8)
#define VisualAllMask		((1 << 9) - 1)

/*
 * This defines a window manager property that clients may use to
 * share standard color maps:
 */

typedef struct
{
	Colormap 	colormap;
	unsigned long 	red_max;
	unsigned long 	red_mult;
	unsigned long 	green_max;
	unsigned long 	green_mult;
	unsigned long 	blue_max;
	unsigned long 	blue_mult;
	unsigned long 	base_pixel;
} XStandardColormap;

/*
 * return codes for XReadBitmapFile and XWriteBitmapFile
 */

#define BitmapSuccess		0
#define BitmapOpenFailed 	1
#define BitmapFileInvalid 	2
#define BitmapNoMemory		3

/*
 * Declare the routines that don't return int.
 */

/****************************************************************
 *
 * Context Management
 *
 ****************************************************************/


/* Associative lookup table return codes */

#define XCSUCCESS 0	/* No error. */
#define XCNOMEM   1     /* Out of memory */
#define XCNOENT   2     /* No entry in table */

typedef int XContext;

#define XUniqueContext()       		((XContext) XrmUniqueQuark())
#define XStringToContext(string)   	((XContext) XrmStringToQuark(string))

extern int XSaveContext( Display * dpy, Window win, XContext context, caddr_t data );
    /* Window       window;		*/
    /* XContext    context;		*/
    /* caddr_t      data;		*/

extern int XFindContext( Display * dpy, Window win, XContext context, caddr_t * data );
    /* Display	    *display;		*/
    /* Window       window;		*/
    /* XContext    context;		*/
    /* caddr_t      *data;  *//* RETURN */

extern int XDeleteContext( Display * dpy, Window win, XContext context );


XWMHints *	XGetWMHints(	Display *, Window );
Region 		XCreateRegion(	void );
Region		XPolygonRegion(	XPoint *, int, int );
XImage *	XCreateImage(	Display *, Visual *, unsigned int, int, int, char *, unsigned int, unsigned int, int, int );

XVisualInfo *	XGetVisualInfo( Display *, long, XVisualInfo *, int * );

#endif /* _XUTIL_H_ */
