/*
 * extensions the Helios General Server Protocol for X
 *
 * by NC
 */
 
/* SccsId: %W%	%G% Copyright (C) 1987, Perihelion Software Ltd.	*/

#ifndef _XGSP_H
#define _XGSP_H

#ifndef __helios_h
#include <helios.h>
#endif

/* -------------------- Select ------------------------------------ */

/*
 * a Select request contains a mask indicating the kind of select
 * and timeout for the maximum delay of the select
 */

typedef struct SelectRequest
{
	word		type;		/* type of select */
	word		timeout;	/* duration of select */
} SelectRequest;

/* the reply indicates the kind of selects that are 'ready' */

typedef struct SelectReply
{
        word		type;
} SelectReply;

/*
 * Select Protocol:
 *
 * Version 0.1
 *
 *
 * 1] The Client sends a FG_Select msg, which includes a single control word 
 *      defining the items on this stream to be selected.
 *    The Header timeout field will define the period of the server select.
 *
 * 2] The Client then waits on the Reply port for a Select Ack. The Ack. may 
 *      include the select mask (items selected) -SelectRc_Done, or may specify
 *      that the Select is now in progress -SelectRc_Wait.
 * 
 * 3] While not Done the Client waits on Reply port FOREVER. It must not timeout
 *      because there is no way of protecting the Reply port from a pre-empting 
 *      Select termination.
 * 
 * 4] If The Select completes: A SelectReply will be returned, with the relevant
 *      select mask.
 * 
 *    Else If the Select Timesout: A SelectReply will be returned, with a Null
 *      select mask (=Select_None).
 * 
 * 5] If the Client wishes to abort the Select it must issue an AbortPort to the
 *     Server port of the stream. This will terminate the Select gracefully(Ha! Ha!)
 *     so that it behaves as if it timed-out. 
 */

/* the select types */

#define Select_None     (0)
#define Select_Read	(1<<0)
#define Select_Write	(1<<1)
#define Select_Mask     ((1<<2) - 1)

/* the return code values */

#define SelectRc_Mask		0x0f
#define SelectRc_Done		0x02
#define SelectRc_Wait		0x01


#endif /* _XGSP_H */

